@@ -524,12 +524,13 @@ void arrayline(char *line)
 					w);
 			else {
 				char *endptr;
-				mis.super_minor= strtol(w+12, &endptr, 10);
-				if (w[12]==0 || endptr[0]!=0 || mis.super_minor < 0) {
+				int minor = strtol(w+12, &endptr, 10);
+
+				if (w[12]==0 || endptr[0]!=0 || minor < 0)
 					fprintf(stderr, Name ": invalid super-minor number: %s\n",
 						w);
-					mis.super_minor = UnSet;
-				}
+				else
+					mis.super_minor = minor;
 			}
 		} else if (strncasecmp(w, "name=", 5)==0) {
 			if (mis.name[0])
@@ -677,12 +678,21 @@ void homehostline(char *line)
 static char *auto_options = NULL;
 void autoline(char *line)
 {
+	char *w;
+
 	if (auto_options) {
 		fprintf(stderr, Name ": AUTO line may only be give once."
 			"  Subsequent lines ignored\n");
 		return;
 	}
-	auto_options = line;		
+
+	auto_options = dl_strdup(line);
+	dl_init(auto_options);
+
+	for (w=dl_next(line); w != line ; w=dl_next(w)) {
+		char *w2 = dl_strdup(w);
+		dl_add(auto_options, w2);
+	}
 }
 
 int loaded = 0;
@@ -882,18 +892,26 @@ int conf_test_dev(char *devname)
 	return 0;
 }
 
-int conf_test_metadata(const char *version)
+int conf_test_metadata(const char *version, int is_homehost)
 {
 	/* Check if the given metadata version is allowed
 	 * to be auto-assembled.
 	 * The default is 'yes' but the 'auto' line might over-ride that.
-	 * Word in auto_options are processed in order with the first
+	 * Words in auto_options are processed in order with the first
 	 * match winning.
 	 * word can be:
 	 *   +version   - that version can be assembled
 	 *   -version   - that version cannot be auto-assembled
 	 *   yes or +all - any other version can be assembled
 	 *   no or -all  - no other version can be assembled.
+	 *   homehost   - any array associated by 'homehost' to this
+	 *                host can be assembled.
+	 *
+	 * Thus:
+	 *   +ddf -0.90 homehost -all
+	 * will auto-assemble any ddf array, no 0.90 array, and
+	 * any other array (imsm, 1.x) if and only if it is identified
+	 * as belonging to this host.
 	 */
 	char *w;
 	load_conffile();
@@ -905,6 +923,12 @@ int conf_test_metadata(const char *version)
 			return 1;
 		if (strcasecmp(w, "no") == 0)
 			return 0;
+		if (strcasecmp(w, "homehost") == 0) {
+			if (is_homehost)
+				return 1;
+			else
+				continue;
+		}
 		if (w[0] == '+')
 			rv = 1;
 		else if (w[0] == '-')
